import numpy as np
import pandas as pd
import altair as alt


def plot_text(X, texts, color_array=None, color_words=None, disable_warning=True):
    """
    Make a visualisation to help find clues in text data.

    Arguments:
        - `X`: the numeric features, should be a 2D numpy array
        - `texts`: list of text data
        - `color_words`: list of words to highlight
        - `disable_warning`: disable the standard altair max rows warning
    """
    if disable_warning:
        alt.data_transformers.disable_max_rows()

    if len(texts) != X.shape[0]:
        raise ValueError(
            f"The number of text examples ({len(texts)}) should match X array ({X.shape[0]})."
        )

    df_ = pd.DataFrame({"x1": X[:, 0], "x2": X[:, 1], "text": texts}).assign(
        trunc_text=lambda d: d["text"].str[:120], r=0
    )

    if color_array is not None:
        if len(color_array) != X.shape[0]:
            raise ValueError(
                f"The number of color array ({len(color_array)}) should match X array ({X.shape[0]})."
            )
        df_ = df_.assign(color=color_array)

    if color_words:
        df_ = df_.assign(color="none")

        for w in color_words:
            predicate = df_["text"].str.lower().str.contains(w)
            df_ = df_.assign(color=lambda d: np.where(predicate, w, d["color"]))

    brush = alt.selection(type="interval")

    p1 = (
        alt.Chart(df_)
        .mark_circle(opacity=0.6, size=20)
        .encode(
            x=alt.X("x1", axis=None, scale=alt.Scale(zero=False)),
            y=alt.Y("x2", axis=None, scale=alt.Scale(zero=False)),
            tooltip=["text"],
        )
        .properties(width=350, height=350, title="embedding space")
        .add_selection(brush)
    )

    if color_words:
        p1 = (
            alt.Chart(df_)
            .mark_circle(opacity=0.6, size=20)
            .encode(
                x=alt.X("x1", axis=None, scale=alt.Scale(zero=False)),
                y=alt.Y("x2", axis=None, scale=alt.Scale(zero=False)),
                tooltip=["text"],
                color=alt.Color("color", sort=["none"] + color_words),
            )
            .properties(width=350, height=350, title="embedding space")
            .add_selection(brush)
        )

    if color_array is not None:
        p1 = (
            alt.Chart(df_)
            .mark_circle(opacity=0.6, size=20)
            .encode(
                x=alt.X("x1", axis=None, scale=alt.Scale(zero=False)),
                y=alt.Y("x2", axis=None, scale=alt.Scale(zero=False)),
                tooltip=["text"],
                color=alt.Color("color"),
            )
            .properties(width=350, height=350, title="embedding space")
            .add_selection(brush)
        )

    p2 = (
        alt.Chart(df_)
        .mark_text()
        .encode(
            x=alt.X("r", axis=None),
            y=alt.Y("row_number:O", axis=None),
            text="trunc_text:N",
        )
        .transform_window(row_number="row_number()")
        .transform_filter(brush)
        .transform_window(rank="rank(row_number)")
        .transform_filter(alt.datum.rank < 18)
        .properties(title="text")
    )

    return (p1 | p2).configure_axis(grid=False).configure_view(strokeWidth=0)



import pathlib
from setuptools import setup, find_packages


base_packages = ["altair>=4.0.1", "numpy>=1.19.2"]

dev_packages = base_packages + [
    "pytest>=4.0.2",
    "black>=19.3b0",
    "pytest-cov>=2.6.1",
    "pre-commit>=2.2.0",
    "umap-learn",
    "jupyterlab",
]


setup(
    name="cluestar",
    version="0.1.0",
    author="Vincent D. Warmerdam",
    packages=find_packages(exclude=["notebooks", "docs"]),
    description="Gain a clue by clustering!",
    long_description=pathlib.Path("README.md").read_text(),
    long_description_content_type="text/markdown",
    url="https://github.com/koaning/cluestar/",
    project_urls={
        "Documentation": "https://github.com/koaning/cluestar/",
        "Source Code": "https://github.com/koaning/cluestar/",
        "Issue Tracker": "https://github.com/koaning/cluestar/issues",
    },
    install_requires=base_packages,
    extras_require={"base": base_packages, "dev": dev_packages},
    classifiers=[
        "Intended Audience :: Science/Research",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "License :: OSI Approved :: MIT License",
        "Topic :: Scientific/Engineering",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
    ],
)
